"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarnManager = void 0;
const discord_js_1 = require("discord.js");
const ModeratorEnums_1 = require("../ModeratorEnums");
/**
 * Class that controls Moderation Warn System
 *
 * @class
 * @classdesc Moderator Warn Manager Class
 */
class WarnManager {
    moderator;
    /**
     * @constructor
     *
     * @param {Moderator} moderator Moderator Class
     */
    constructor(moderator) {
        /**
         * Moderator Class
         *
         * @type {Moderator}
         * @private
         */
        this.moderator = moderator;
    }
    /**
     * Method for issuing a warning to a Discord guild user
     *
     * @param {CreateWarnOptions} options Create warn options
     *
     * @returns {Promise<GuildWarn | DefaultObject>} Returns warning data or an object with an error
     */
    add(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.moderator.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.moderator.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            this.moderator.utils.fetchGuild(guild.id);
            const guildData = this.moderator.database.get(guild.id);
            const guildMemberWarns = guildData?.warns?.filter(warn => warn.users.target === member.id);
            const guildMemberActiveWarns = guildMemberWarns.filter(guildMemberWarn => !guildMemberWarn.unwarnedAt);
            const defaultWarnData = {
                id: this.moderator.utils.generateID(),
                channelID: options.channel instanceof discord_js_1.TextChannel ? options.channel.id : options.channel,
                users: {
                    target: member.id,
                    moderator: options.moderator ?? this.moderator.client.user.id
                },
                reason: options.reason ?? null,
                warnedAt: Date.now(),
                unwarnedAt: null
            };
            guildData.warns.push(defaultWarnData);
            this.moderator.database.set(member.guild.id, guildData);
            this.moderator.emit(ModeratorEnums_1.Events.WARN_ADD, member.guild, defaultWarnData);
            if (this.moderator.options.managers.warn.punishment.isEnabled) {
                const punishment = this.moderator.utils.fetchPunishment(guildMemberActiveWarns.length + 1);
                if (punishment)
                    await this.moderator.punish({
                        type: punishment,
                        user: member,
                        guild: guild,
                        channel: options.channel,
                        time: punishment.includes(ModeratorEnums_1.PunishmentType.TEMPMUTE) ? this.moderator.options.managers.warn.punishment.time : 0,
                        reason: options.reason ?? null,
                        moderator: options.moderator ?? this.moderator.client.user.id,
                    });
            }
            return res(defaultWarnData);
        });
    }
    /**
     * Method for getting warnings data
     *
     * @param {FetchOptions} options Fetch options
     *
     * @returns {Promise<GuildWarn | DefaultObject>} Returns information about a warning or an object with an error
     */
    get(options) {
        return new Promise(async (res, rej) => {
            if (!options.guild && !options.user) {
                let userWarn = null;
                this.moderator.database.forEach(guildData => {
                    const guildUserWarn = guildData.warns.find(guildWarn => guildWarn.id === options.id);
                    if (guildUserWarn?.id)
                        userWarn = guildUserWarn;
                });
                return res(userWarn);
            }
            else {
                const guildData = this.moderator.database.get(options.guild);
                if (!guildData)
                    return res({
                        status: false,
                        message: `Discord guild with ID '${options.guild}' not found in database!`
                    });
                const guildMemberWarn = guildData.warns.find(guildWarn => guildWarn.id && guildWarn.users.target === options.user);
                if (!guildMemberWarn)
                    return res({
                        status: false,
                        message: ``
                    });
                return res(guildMemberWarn);
            }
        });
    }
    /**
     * A method to get a list of warnings for a guild or user
     *
     * @param {ListOptions} options Fetch options
     *
     * @returns {Promise<Array<GuildWarn>>} Returns a list of warnings
     */
    list(options) {
        return new Promise(async (res, rej) => {
            this.moderator.utils.fetchGuild(options.guild);
            const guildData = this.moderator.database.get(options.guild);
            if (options.user) {
                const guildMemberWarns = guildData?.warns?.filter(guildWarn => guildWarn.users.target === options.user);
                return res(guildMemberWarns || []);
            }
            else
                return res(guildData?.warns || []);
        });
    }
    /**
     * A method for removing warnings from a user in the Discord guild
     *
     * @param {DeleteWarnOptions} options Delete warn options
     *
     * @returns {Promise<GuildWarn | DefaultObject>} Returns altered warning data or an object with an error
     */
    delete(options) {
        return new Promise(async (res, rej) => {
            let guild = null;
            let member = null;
            if (options.guild instanceof discord_js_1.Guild)
                guild = options.guild;
            else
                guild = await this.moderator.fetcher.guild(options.guild);
            if (!guild)
                return res({
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                });
            if (options.user instanceof discord_js_1.GuildMember)
                member = options.user;
            else
                member = await this.moderator.fetcher.member(guild.id, options.user);
            if (!member)
                return res({
                    status: false,
                    message: `The user with ID '${options.user instanceof discord_js_1.GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof discord_js_1.Guild ? options.guild.id : options.guild}'!`
                });
            this.moderator.utils.fetchGuild(guild.id);
            const guildData = this.moderator.database.get(guild.id);
            const guildMemberWarns = guildData?.warns?.filter(warn => warn.users.target === member.id);
            const guildMemberActiveWarns = guildMemberWarns?.filter(guildMemberWarn => !guildMemberWarn.unwarnedAt);
            if (!guildMemberActiveWarns.length)
                return res({
                    status: false,
                    message: `User with ID '${member.id}' has no active warnings!`
                });
            const guildMemberWarn = guildMemberActiveWarns.find(guildMemberActiveWarn => guildMemberActiveWarn.id === options.id);
            if (!guildMemberWarn)
                return res({
                    status: false,
                    message: `Warnings with ID '${options.id}' not found with user with ID '${member.id}' in Discord guild with ID '${guild.id}'!`
                });
            guildMemberWarn.unwarnedAt = Date.now();
            guildMemberActiveWarns.filter(guildMemberActiveWarn => guildMemberActiveWarn.id != options.id);
            guildMemberActiveWarns.push(guildMemberWarn);
            this.moderator.database.set(member.guild.id, guildData);
            return res(guildMemberWarn);
        });
    }
}
exports.WarnManager = WarnManager;
