import Enmap from "enmap";
import { Client, Guild, GuildMember, TextChannel, TextChannelResolvable } from "discord.js";

import { ModeratorEmitter } from "./ModeratorEmitter";

import { MuteManager } from "./managers/MuteManager";
import { WarnManager } from "./managers/WarnManager";
import { FetchManager } from "./managers/FetchManager";
import { UtilsManager } from "./managers/UtilsManager";

import { ModeratorOptions } from "../types/ModeratorOptions";

import { ClearData, DefaultObject } from "../types/ModeratorData";
import { ModeratorGuildData, ModeratorUsers } from "../types/ModeratorDatabase";

import { Events, MuteType, PunishmentType } from "./ModeratorEnums";

import { author, version, homepage } from "../package.json";
import { BanOptions, ClearOptions, KickOptions, Options, PunishOptions, UnbanOptions } from "../types/Options";

/**
 * Class that controls Moderation System
 * 
 * @class
 * @classdesc Moderator Class
 * @extends {ModeratorEmitter}
 */
export class Moderator extends ModeratorEmitter {
    public client: Client;
    public database: Enmap<string, ModeratorGuildData>;

    public author: string;
    public website: string;
    public version: string;

    public ready: boolean;
    public readyTimestamp: number;
    public options: ModeratorOptions;

    public mutes: MuteManager | null;
    public warns: WarnManager | null;
    public fetcher: FetchManager | null;
    public utils: UtilsManager;

    /**
     * @constructor
     *
     * @param {Client} client Discord Client
     * @param {ModeratorOptions} [options] Module Options
     */
    constructor(client: Client, options?: ModeratorOptions) {
        super();
        
        /**
         * Discord Client
         * 
         * @type {Client}
         * @private
         */
        this.client = client;

        /**
         * Moderator Database
         * 
         * @type {?Enmap}
         */
        this.database = null;

        /**
         * Moderator Author
         * 
         * @type {string}
         */
        this.author = author;

        /**
         * Moderator WebSite URL
         * 
         * @type {string}
         */
        this.website = homepage;

        /**
         * Moderator Version
         * 
         * @type {string}
         */
        this.version = version;

        /**
         * Moderator Ready Status
         * 
         * @type {?boolean}
         */
        this.ready = null;

        /**
         * Moderator Options
         * 
         * @type {?ModeratorOptions}
         */
        this.options = null;

        /**
         * Moderator Mute Manager
         * 
         * @type {?MuteManager}
         */
        this.mutes = null;

        /**
         * Moderator Warn Manager
         * 
         * @type {?WarnManager}
         */
        this.warns = null;

        /**
         * Moderator Fetch Manager
         * 
         * @type {?FetchManager}
         */
        this.fetcher = null;

        /**
         * Moderator Utils Manager
         * 
         * @type {UtilsManager}
         */
        this.utils = new UtilsManager(this);
        
        this.init(options);
    }

    /**
     * Method for kicking Discord guild users
     * 
     * @param {KickOptions} options Kick options
     * 
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    public kick(options: Options<KickOptions>): Promise<DefaultObject> {
        return new Promise(async (res, rej) => {
            let guild: Guild = null;
            let member: GuildMember = null;
            
            if(options.guild instanceof Guild) guild = options.guild;
            else guild = await this.fetcher.guild(options.guild);

            if(!guild) return res(
                {
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                }
            )

            if(options.user instanceof GuildMember) member = options.user;
            else member = await this.fetcher.member(guild.id, options.user);

            if(!member) return res(
                {
                    status: false,
                    message: `The user with ID '${options.user instanceof GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof Guild ? options.guild.id : options.guild}'!`
                }
            )

            if(!member.kickable) return res(
                {
                    status: false,
                    message: `A target with ID '${member.id}' cannot be kicked!`
                }
            )

            try {
                await member.kick(options.reason);
            } catch(error) {
                return res(
                    {
                        status: false,
                        message: error.message
                    }
                )
            }

            this.emit(Events.KICK, member.guild, member);

            return res({ status: true });
        })
    }

    /**
     * Method for banning Discord guild users
     * 
     * @param {BanOptions} options Ban options
     * 
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    public ban(options: Options<BanOptions>): Promise<DefaultObject> {
        return new Promise(async (res, rej) => {
            let guild: Guild = null;
            let member: GuildMember = null;
            
            if(options.guild instanceof Guild) guild = options.guild;
            else guild = await this.fetcher.guild(options.guild);

            if(!guild) return res(
                {
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                }
            )

            if(options.user instanceof GuildMember) member = options.user;
            else member = await this.fetcher.member(guild.id, options.user);

            if(!member) return res(
                {
                    status: false,
                    message: `The user with ID '${options.user instanceof GuildMember ? options.user.id : options.user}' is not found in the Discord guild with ID '${options.guild instanceof Guild ? options.guild.id : options.guild}'!`
                }
            )

            if(!member.bannable) return res(
                {
                    status: false,
                    message: `The target with ID '${member.id}' cannot be banned!`
                }
            )

            try {
                await member.ban({ reason: options.reason });
            } catch(error) {
                return res(
                    {
                        status: false,
                        message: error.message
                    }
                )
            }

            this.emit(Events.BAN, member.guild, member);

            return res({ status: true });
        })
    }

    /**
     * Method for removing bans from Discord guild users
     * 
     * @param {UnbanOptions} options Unban options
     * 
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    public unban(options: Options<UnbanOptions>): Promise<DefaultObject> {
        return new Promise(async (res, rej) => {
            let guild: Guild = null;

            if(options.guild instanceof Guild) guild = options.guild;
            else guild = await this.fetcher.guild(options.guild);

            if(!guild) return res(
                {
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                }
            )

            const guildBans = await guild.bans.fetch();

            if(!guildBans || !guildBans?.size) return res(
                {
                    status: false,
                    message: `Bans not found in Discord guild with ID '${options.guild}'!`
                }
            )

            const guildMemberBan = guildBans.find(guildBan => guildBan.user.id === options.user);
            
            if(!guildMemberBan) return res(
                {
                    status: false,
                    message: `The target with ID '${options.user}' has no bans on the server with ID '${options.guild}'!`
                }
            )

            try {
                await guild.members?.unban(guildMemberBan.user);
            } catch(error) {
                return res(
                    {
                        status: false,
                        message: error.message
                    }
                )
            }

            this.emit(Events.UNBAN, guildMemberBan.guild, guildMemberBan.user);

            return res({ status: true });
        })
    }

    /**
     * Method for punishing Discord guild users
     * 
     * @param {PunishOptions} options Punish options
     * 
     * @returns {Promise<DefaultObject>} Returns the state of the action or an object with an error
     */
    public punish(options: Options<PunishOptions>): Promise<DefaultObject> {
        return new Promise(async (res, rej) => {
            let isSuccess = false;
            let guild: Guild = null;
            let member: GuildMember = null;

            if(options.guild instanceof Guild) guild = options.guild;
            else guild = await this.fetcher.guild(options.guild);

            if(!guild) return res(
                {
                    status: false,
                    message: `Discord guild with ID '${options.guild}' is not found in the client!`
                }
            )

            if(options.user instanceof GuildMember) member = options.user;
            else member = await this.fetcher.member(guild.id, options.user);

            switch(options.type) {
                case PunishmentType.BAN: {
                    await this.ban(
                        {
                            user: member,
                            guild: guild,
                            reason: options.reason
                        }
                    )

                    isSuccess = true;

                    break;
                }

                case PunishmentType.KICK: {
                    await this.kick(
                        {
                            user: member,
                            guild: member.guild.id,
                            reason: options.reason
                        }
                    )

                    isSuccess = true;

                    break;
                }

                case PunishmentType.MUTE: {
                    await this.mutes?.create(MuteType.PERMANENT, {
                        reason: options.reason,
                        moderator: options.moderator,

                        member: member,
                        channel: options.channel
                    })

                    isSuccess = true;

                    break;
                }

                case PunishmentType.TEMPMUTE: {
                    await this.mutes?.create(MuteType.PERMANENT, {
                        time: options.time,
                        reason: options.reason,
                        moderator: options.moderator,

                        member: member,
                        channel: options.channel
                    })

                    isSuccess = true;

                    break;
                }
            }

            return res({ status: isSuccess });
        })
    }

    /**
     * Method for deleting messages in text channels
     * 
     * @param {ClearOptions} options Clear options
     * 
     * @returns {Promise<ClearData | DefaultObject>} Returns the number of deleted messages or an object with an error
     */
    public clear(options: Options<ClearOptions>): Promise<ClearData | DefaultObject> {
        return new Promise(async (res, rej) => {
            if(new Object(options).hasOwnProperty("count")) options.count = 100;

            if(options.count < 1 || options.count > 100) return res(
                {
                    status: false,
                    message: `The specified number of '${options.count}' messages violates the Discord limit of 0 to 100 messages!`
                }
            )

            let channel: TextChannel = null;

            if(options.channel instanceof TextChannel) channel = options.channel;
            else channel = await this.fetcher.channel(options.channel) as TextChannel;

            try {
                const deletedMessages = await channel.bulkDelete(options.count, true);

                return res(
                    {
                        count: options.count,
                        deleted: deletedMessages.size
                    }
                )
            } catch(error) {
                return res(
                    {
                        status: false,
                        message: error.message
                    }
                )
            }
        })
    }

    /**
     * Method for module initialization
     * 
     * @param {ModeratorOptions} [options={}] Moderator Options
     * 
     * @private
     */
    private init(options?: ModeratorOptions): void {
        const interval = setInterval(async () => {
            if(this.client.isReady()) {
                this.utils.fetchIntents();

                this.ready = true;
                this.readyTimestamp = Date.now();
                this.options = this.utils.fetchOptions(options ?? {});

                this.initDatabase();
                this.initManagers();

                this.emit(Events.READY, this);

                clearInterval(interval);
            }
        }, 100)
    }

    /**
     * Method for initializing the module database
     * 
     * @private
     */
    private initDatabase(): void {
        this.database = new Enmap({
            name: this.options.database.name,
            dataDir: this.options.database.path,
            wal: false
        })
    }

    /**
     * Method for initializing all module managers
     * 
     * @private
     */
    private initManagers(): void {
        if(this.options.managers.mute.isEnabled) this.mutes = new MuteManager(this);
        if(this.options.managers.warn.isEnabled) this.warns = new WarnManager(this);

        this.fetcher = new FetchManager(this);
    }
}

/**
 * @typedef {object} ModeratorOptions
 * 
 * @prop {DatabaseConfig} [database] Moderator Database Configuration
 * @prop {ModeratorManagers} [managers] Moderator Managers Configuration
 */

/**
 * @typedef {object} DatabaseConfig
 * 
 * @prop {string} [path="./"] Database Path
 * @prop {string} [name="moderator"] Database Name
 */

/**
 * @typedef {object} ModeratorManagers
 * 
 * @prop {MuteManagerConfig} [mute] Moderator Mute Manager
 * @prop {WarnManagerConfig} [warn] Moderator Warn Manager
 */

/**
 * @typedef {object} MuteManagerConfig
 * 
 * @prop {boolean} [isEnabled=true] Mute Manager State
 * @prop {number} [interval=5000] Mute Manager Check Interval
 */

/**
 * @typedef {object} WarnManagerConfig
 * 
 * @prop {boolean} [isEnabled=true] Warn Manager State
 * @prop {number | [number, number]} [limits=[3, 6]] Warn Manager Limits
 * @prop {WarnPunishmentConfig} [punishment] Warn Manager Punishment
 */

/**
 * @typedef {object} WarnPunishmentConfig
 * 
 * @prop {boolean} [isEnabled=true] Warn Manager Punishments State
 * @prop {number} [time=86400] Warn Manager Mute Time
 * @prop {PunishmentType | [PunishmentType, PunishmentType]} [type] Warn Manager Punishments Type
 */

/**
 * @typedef {object} ModeratorGuildData
 * 
 * @prop {string} guildID Discord Guild ID
 * @prop {string} muteRole Discord Guild Mute Role ID
 * @prop {Array<GuildMute>} mutes Discord Guild Mutes Array
 * @prop {Array<GuildWarn>} warns Discord Guild Warns Array
 */

/**
 * @typedef {object} GuildMute
 * 
 * @prop {string} guildID Discord Guild ID
 * @prop {string} channelID Discord Guild Channel ID
 * @prop {ModeratorUsers} users Mute Users
 * @prop {number} time Mute Time
 * @prop {string} reason Mute Reason
 * @prop {number} mutedAt Date and time of mute in milliseconds
 * @prop {number} unmutedAt Date and time of mute removal in milliseconds
 */

/**
 * @typedef {object} ModeratorUsers
 * 
 * @prop {string} target Mute Target ID
 * @prop {string} moderator Mute Moderator ID
 */

/**
 * @typedef {object} DefaultObject
 * 
 * @prop {boolean} status Status of a specific operation
 * @prop {string} [message] Text of the error, if any
 */

/**
 * @typedef {object} CreationMuteOptions
 * 
 * @prop {number} [time] Mute Time
 * @prop {string} [reason] Mute Reason
 * @prop {string} [moderator] Mute Moderator ID
 * @prop {GuildMember} member Mute Target
 * @prop {TextChannelResolvable} channel Guild Text Channel Resolvable
 */